#include "TValHashMapT.hh"
#include "TDBConnectionManager.hh"
#include "MovieTable.hh"


void
MovieTable::Select (UserTable& utable, FormatTable& ftable,
		    bool sort_by_insert_order)
{
  TDBConnection *conn;
  TDBResult *res;
  MovieRow *row;
  TString sql ("SELECT * FROM movie ");
  int tuples, i, j, k;
  bool second = false;
  
  /* Remove all old rows from vector. */
  rows.ClearAndDestroy ();
  
  /* Build sql statement. */
  if (! utable.IsToDisplayAll ())
    {
      sql += "WHERE owner IN (";
      
      j = utable.rows.Entries ();
      k = 0;
      for (i = 0; i < j; i++)
	if (utable.rows[i]->display)
	  {
	    if (k)
	      sql += ",";
	    
	    sql += utable.rows[i]->id;
	    k++;
	  }

      /* Quit if user selection is empty. */
      if (! k)
	return;
      
      sql += ") ";
      second = true;
    }
  
  if (! ftable.IsToDisplayAll ())
    {
      if (second)
	sql += "AND format IN (";
      else
	sql += "WHERE format IN (";
      
      j = ftable.rows.Entries ();
      k = 0;
      for (i = 0; i < j; i++)
	if (ftable.rows[i]->display)
	  {
	    if (k)
	      sql += ",";
	    
	    sql += ftable.rows[i]->id;
	    k++;
	  }
      
      /* Quit if format selection is empty. */
      if (! k)
	return;
      
      sql += ") ";
    }
  
  if (sort_by_insert_order)
    sql += "ORDER BY id DESC";
  else
    sql += "ORDER BY title ASC";
  
  conn = TDBConnectionManager::Connection ();
  if (conn->IsValid ())
    {
      res = conn->Execute (sql);
      if (res->IsValid ())
	{
	  tuples = res->Tuples ();
	  for (int i = 0; i < tuples; i++)
	    {
	      row = new MovieRow ();
	      row->id        = res->Value (i, 0);
	      row->title     = res->Value (i, 1);
	      row->notes     = res->Value (i, 2);
	      row->format_id = res->Value (i, 3);
	      row->owner_id  = res->Value (i, 4);
	      row->cuser_id  = res->Value (i, 5);
	      rows.Append (row);
	    }
	  
	  SelectedRowsMessage (tuples);
	}
      else
	myErrorMessages.Append (res->ErrorMessage ());
      
      delete res;
    }
  else
    myErrorMessages.Append (conn->ErrorMessage ());
  
  /* Look up the format and user names. */
  j = rows.Entries ();
  for (i = 0; i < j; i++)
    {
      if (utable.map.FindValue (rows[i]->owner_id, k))
	{
	  rows[i]->owner = utable.rows[k]->name;
	  rows[i]->owner_index = k;
	}
      
      if (utable.map.FindValue (rows[i]->cuser_id, k))
	{
	  rows[i]->cuser = utable.rows[k]->name;
	  rows[i]->cuser_index = k;
	}
      
      if (ftable.map.FindValue (rows[i]->format_id, k))
	{
	  rows[i]->format = ftable.rows[k]->name;
	  rows[i]->format_index = k;
	}
    }
}


void
MovieTable::Delete (int row)
{
  TString sql ("DELETE FROM movie WHERE id=");
  
  sql += rows[row]->id;
  
  Execute (sql);
}


void
MovieTable::Update (MovieRow& mrow)
{
  TString sql ("UPDATE movie SET ");
  
  sql += "title='";
  sql += QuoteString (mrow.title);
  sql += "', notes='";
  sql += QuoteString (mrow.notes);
  sql += "', format=";
  sql += mrow.format_id;
  sql += ", owner=";
  sql += mrow.owner_id;
  sql += ", cuser=";
  sql += mrow.cuser_id;
  sql += " WHERE id=";
  sql += mrow.id;
  
  Execute (sql);
}


void
MovieTable::Insert (MovieRow& mrow)
{
  TString sql ("INSERT INTO movie (title,notes,format,owner,cuser) VALUES ('");
  
  sql += QuoteString (mrow.title);
  sql += "', '";
  sql += QuoteString (mrow.notes);
  sql += "', ";
  sql += mrow.format_id;
  sql += ", ";
  sql += mrow.owner_id;
  sql += ", ";
  sql += mrow.cuser_id;
  sql += ")";
  
  Execute (sql);
}
