#ifndef _TABLE_HH
#define _TABLE_HH

#include <TString.hh>
#include <TValListT.hh>
#include <THashString.hh>


/**
 * Base class for database tables.
 */

class Table
{
public:
  
  virtual ~Table () {}
  
  /**
   * Delete the selected row from the database.
   */
  virtual void Delete (int row) = 0;
  
  /**
   * Execute simple SQL statement.
   * No result can be obtained.
   */
  void Execute (TString &sql);
  
  /**
   * Send number of selected rows to status bar in UI.
   */
  void SelectedRowsMessage (int n);
  
  /**
   * Duplicate quotes in string so that one can use it in a SQL
   * statement. Example: ' turns into ''.
   *
   * @param str  String to quote.
   * @return     Converted string.
   */
  TString QuoteString (TString& str);
  
  /**
   * See if there are any errors in the error message list.
   *
   * @return  true, there are errors or false, no errors.
   */
  bool HasErrors () { return myErrorMessages.Entries (); }
  
  /**
   * Get top error of list.
   *
   * @return  Top error message.
   */
  TString PopError ();
  
protected:
  
  //! Error messages that has occured
  TValListT<TString> myErrorMessages;
};

#endif // _MOVIE_TABLE_HH
