#include "UiDialog.hh"

#include <stdio.h>


UiDialog::UiDialog (bool modal, int buttons, int default_button)
  : Gtk::Window (GTK_WINDOW_DIALOG),
    myOkPressed (false),
    myIsBlocked (false)
{
  Gtk::HBox *hbox;
  Gtk::HSeparator *hsep;
  Gtk::Button *ok_button, *cancel_button;
  
  set_border_width (5);
  set_position (GTK_WIN_POS_CENTER);

  /* Add our VBOX to the window. */
  add (myVBox);
  
  /* A HBOX to put the buttons in. */
  hbox = manage (new Gtk::HBox (true, 0));
  
  hsep = manage (new Gtk::HSeparator ());
  
  myVBox.pack_end (*hbox);
  myVBox.pack_end (*hsep, false, false, 5);
  
  /* Create the buttons. */
  if (buttons >= 1)
    {
      ok_button = manage (new Gtk::Button ("OK"));
      ok_button->clicked.connect(bind<int>(slot(this, &UiDialog::Ok), 0));
      ok_button->set_flags (GTK_CAN_DEFAULT);
      hbox->pack_start (*ok_button, true, true, 20);
      if (default_button == 1)
	ok_button->grab_default ();
      ok_button->show ();
    }
  
  if (buttons >= 2)
    {
      cancel_button = manage (new Gtk::Button ("Cancel"));
      cancel_button->clicked.connect
	(bind<int>(slot(this, &UiDialog::Cancel), 0));
      cancel_button->set_flags (GTK_CAN_DEFAULT);
      hbox->pack_start (*cancel_button, true, true, 20);
      if (default_button == 2)
	cancel_button->grab_default ();
      cancel_button->show ();
    }
  
  myVBox.show ();
  hsep->show ();
  hbox->show ();
  
  if (modal)
    set_modal (true);
  
  /* Use show() to display the window! */
}


void
UiDialog::Ok (int dummy)
{
  myOkPressed = true;
  hide ();

  if (myIsBlocked)
    {
      myIsBlocked = false;
      Gtk::Main::quit ();
    }
}


void
UiDialog::Cancel (int dummy)
{
  myOkPressed = false;
  hide ();
  
  if (myIsBlocked)
    {
      myIsBlocked = false;
      Gtk::Main::quit ();
    }
}


gint
UiDialog::delete_event_impl (GdkEventAny*)
{
  myOkPressed = false;
  hide ();
  
  if (myIsBlocked)
    {
      myIsBlocked = false;
      Gtk::Main::quit ();
    }
  
  /* Don't destroy the window! */
  return 1;
}
