#ifndef _UI_DIALOG_HH
#define _UI_DIALOG_HH

#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/main.h>
#include <gtk--/separator.h>
#include <gtk--/window.h>

using SigC::bind;
using SigC::slot;


/**
 * Base class implementing a simple dialog window.
 * Contains a VBOX with these compoments:
 * - Separator.
 * - Ok and Cancel buttons.
 */

class UiDialog : public Gtk::Window
{
public:
  
  /**
   * Construct dialog widget.
   *
   * @param modal           If the dialog is modal.
   * @param buttons         Number of buttons (Ok and Cancel).
   * @param default_button  Button that is the default.
   */
  UiDialog (bool modal = true, int buttons = 2, int default_button = 1);
  
  /**
   * Query if OK was pressed.
   */
  inline bool OkPressed () { return myOkPressed; }
  
  /**
   * Wait for dialog to finish.
   */
  inline void Block ()
  {
    myIsBlocked = true;
    Gtk::Main::run ();
  }

  /**
   * Set title of window.
   */
  inline void SetTitle (const char *title) { set_title (title); }
  
  /* If you extend this class,
     pack your widget to the start of this VBOX. */
  Gtk::VBox myVBox;
  
private:
  
  /**
   * Ok button callback.
   */
  virtual void Ok (int dummy);
  
  /**
   * Cancel button callback.
   */
  virtual void Cancel (int dummy);
  
  /**
   * Hide window on delete.
   */
  gint delete_event_impl (GdkEventAny*);
  
  
  /* OK pressed? */
  bool myOkPressed;
  
  /* Are we blocked? */
  bool myIsBlocked;
};

#endif // _UI_DIALOG_HH
