#include "UiMain.hh"
#include "UiList.hh"
#include "UiMovieForm.hh"
#include "UiQuestionBox.hh"
#include "pixmaps.h"

using SigC::slot;
using SigC::bind;

/* Our statusbar */
Gtk::Statusbar* UiMain::myStatusbar = 0;


void destroy_cb (void)
{
  Gtk::Main::quit();
}


UiMain::UiMain ()
  : Gtk::Window (GTK_WINDOW_TOPLEVEL),
    myMovieList (5),
    myUserList (3),
    myFormatList (3),
    mySortByInsertOrder (false)
{
  Gtk::VBox *vbox;
  Gtk::MenuBar *menu_bar;
  Gtk::ScrolledWindow *scrolled_win1, *scrolled_win2, *scrolled_win3;
  
  set_title ("MovieDB Version " VERSION);
  set_usize (750, 550);
  
  myMovieForm.SetTitle ("Movie Form");
  myUserForm.SetTitle ("User Form");
  myFormatForm.SetTitle ("Format Form");
  myAboutWindow.SetTitle ("About");
  
  /* A vbox to put the menu and notebook in. */
  vbox = manage (new Gtk::VBox (false, 0));
  add (*vbox);
  
  /* Menu */
  menu_bar = manage (new Gtk::MenuBar ());
  
  /* Create the file menu */
  Gtk::Menu *menu_file = manage (new Gtk::Menu ());
  menu_file->items().push_back (Gtk::Menu_Helpers::MenuElem
				("_Quit", "<control>Q", slot (destroy_cb)));
  
  /* Create the edit menu */
  Gtk::Menu *menu_edit = manage (new Gtk::Menu ());
  Gtk::Menu_Helpers::MenuList& list_edit = menu_edit->items ();

  list_edit.push_back
    (Gtk::Menu_Helpers::MenuElem
     ("_Update", "<control>U", bind<int> (slot (this, &UiMain::Update), 0)));

  list_edit.push_back
    (Gtk::Menu_Helpers::MenuElem
     ("_Insert", "<control>I", bind<int> (slot (this, &UiMain::Insert), 0)));
  
  list_edit.push_back
    (Gtk::Menu_Helpers::MenuElem
     ("_Delete", "<control>D", bind<int> (slot (this, &UiMain::Delete), 0)));
  
  list_edit.push_back (Gtk::Menu_Helpers::SeparatorElem());
  
  list_edit.push_back
    (Gtk::Menu_Helpers::MenuElem
     ("_Refresh", "<control>R", bind<int> (slot (this, &UiMain::Select), 0)));

  list_edit.push_back (Gtk::Menu_Helpers::SeparatorElem());
  
  list_edit.push_back
    (Gtk::Menu_Helpers::MenuElem
     ("_Toggle Selected", "<control>T",
      bind<int> (slot (this, &UiMain::Toggle), 0)));
  
  /* Create the preferences menu */
  Gtk::Menu *menu_cust = manage (new Gtk::Menu ());
  Gtk::Menu_Helpers::MenuList& list_cust = menu_cust->items ();
  
  list_cust.push_back
    (Gtk::Menu_Helpers::CheckMenuElem
     ("Sort Movies By Insert Order",
      bind<int> (slot (this, &UiMain::SortOrder), 0)));
  
  /* Create the help menu */
  Gtk::Menu* menu_help = manage (new Gtk::Menu ());
  
  Gtk::Menu_Helpers::MenuList& list_help = menu_help->items ();
  
  list_help.push_back
    (Gtk::Menu_Helpers::MenuElem
     ("_About", "<control>A", bind<int> (slot (this, &UiMain::About), 0)));
  
  
  /* Create the menu bar */
  Gtk::Menu_Helpers::MenuList& list_bar = menu_bar->items ();

  list_bar.push_front (Gtk::Menu_Helpers::MenuElem
		       ("_Help","<control>H",*menu_help));
  
  list_bar.front()->right_justify();
  
  menu_bar->items().push_front (Gtk::Menu_Helpers::MenuElem
				("_Customise","<control>C", *menu_cust));
  
  menu_bar->items().push_front (Gtk::Menu_Helpers::MenuElem
				("_Edit","<control>E", *menu_edit));
  
  menu_bar->items().push_front (Gtk::Menu_Helpers::MenuElem
				("_File","<control>F", *menu_file));
  
  /* Initialise the lists */
  myMovieList.set_column_title (0, "Title");
  myMovieList.column(0).set_min_width (100);
  myMovieList.set_column_title (1, "Format");
  myMovieList.column(1).set_min_width (100);
  myMovieList.set_column_title (2, "Owner");
  myMovieList.column(2).set_min_width (100);
  myMovieList.set_column_title (3, "Used By");
  myMovieList.column(3).set_min_width (100);
  myMovieList.set_column_title (4, "Comments");
  myMovieList.column(4).set_min_width (100);
  
  for (int i = 0; i < 5; i++)
    myMovieList.column(i).set_auto_resize (true);
  
  myUserList.set_column_title (0, "Name"); 
  myUserList.column(0).set_min_width (100);
  myUserList.set_column_title (1, "Rows In Movies Tab"); 
  myUserList.column(1).set_min_width (100);
  myUserList.column(1).set_justification (GTK_JUSTIFY_CENTER);
  myUserList.set_column_title (2, "Display In Movies Tab");
  myUserList.column(2).set_min_width (100);
  
  for (int i = 0; i < 3; i++)
    myUserList.column(i).set_auto_resize (true);
  
  myFormatList.set_column_title (0, "Name");
  myFormatList.column(0).set_min_width (100);
  myFormatList.set_column_title (1, "Rows In Movies Tab"); 
  myFormatList.column(1).set_min_width (100);
  myFormatList.column(1).set_justification (GTK_JUSTIFY_CENTER);
  myFormatList.set_column_title (2, "Display In Movies Tab");
  myFormatList.column(2).set_min_width (100);
  
  for (int i = 0; i < 3; i++)
    myFormatList.column(i).set_auto_resize (true);
  
  /* Put lists in scrolled windows. */
  scrolled_win1 = manage (new Gtk::ScrolledWindow ());
  scrolled_win2 = manage (new Gtk::ScrolledWindow ());
  scrolled_win3 = manage (new Gtk::ScrolledWindow ());
  
  scrolled_win1->set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  scrolled_win2->set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  scrolled_win3->set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  
  scrolled_win1->add (myMovieList);
  scrolled_win2->add (myUserList);
  scrolled_win3->add (myFormatList);
  
  /* Initialise the notebook */
  myNotebook.set_tab_pos (GTK_POS_TOP);
  
  myNotebook.pages().push_back
    (Gtk::Notebook_Helpers::TabElem (*scrolled_win1, "Movies"));
  
  myNotebook.pages().push_back
    (Gtk::Notebook_Helpers::TabElem (*scrolled_win2, "Users"));
  
  myNotebook.pages().push_back
    (Gtk::Notebook_Helpers::TabElem (*scrolled_win3, "Formats"));
  
  /* Create the statusbar */
  myStatusbar = manage (new Gtk::Statusbar ());
  
  /* Pack stuff! */
  vbox->pack_start (*menu_bar, false, false);
  vbox->pack_start (myNotebook);
  vbox->pack_start (*myStatusbar, false, false);
  
  /* Pixmaps */
  realize ();
  
  Gdk_Color tmp_colour;
  
  myOkPixmap.create_from_xpm_d (get_window(), myOkBitmap,
				tmp_colour, ok_xpm);

  myCancelPixmap.create_from_xpm_d (get_window(), myCancelBitmap,
				    tmp_colour, cancel_xpm);
  
  /* Base class pointers. */
  myUiLists[0] = &myMovieList;
  myUiLists[1] = &myUserList;
  myUiLists[2] = &myFormatList;
  
  myTables[0] = &myMovieTable;
  myTables[1] = &myUserTable;
  myTables[2] = &myFormatTable;
  
  show_all ();
}


void
UiMain::LoadData ()
{
  int entries, count, i;
  TValHashMapT<TString,int,THashString> user_movie_count;
  TValHashMapT<TString,int,THashString> format_movie_count;
  Gdk_Color colours[2];
  const gchar *row[6] = { 0 }, *empty = "";
  gchar buf[16];
  
  /* Light Grey */
  int lum = 243 << 8 | 243;
  colours[0].set_rgb (lum, lum, lum);
  
  /* White */
  colours[1].set_rgb (65535, 65535, 65535);
  
  myUserTable.Select ();
  DisplayTableErrors (&myUserTable);
  
  myFormatTable.Select ();
  DisplayTableErrors (&myFormatTable);
  
  myMovieTable.Select (myUserTable, myFormatTable, mySortByInsertOrder);
  DisplayTableErrors (&myMovieTable);
  
  /* Freeze lists. */
  myMovieList.freeze ();
  myUserList.freeze ();
  myFormatList.freeze ();
  
  /* Clear all UI lists */
  myMovieList.Clear ();
  myUserList.Clear ();
  myFormatList.Clear ();
  
  entries = myMovieTable.rows.Entries ();
  for (i = 0; i < entries; i++)
    {
      row[0] = myMovieTable.rows[i]->title;
      row[1] = myMovieTable.rows[i]->format;
      row[2] = myMovieTable.rows[i]->owner;
      
      if (myMovieTable.rows[i]->owner ==
	  myMovieTable.rows[i]->cuser)
	row[3] = empty;
      else
	row[3] = myMovieTable.rows[i]->cuser;
      
      row[4] = myMovieTable.rows[i]->notes;
      myMovieList.Append (row);
      
      /* Update the number of movies per user count. */
      count = 0;
      user_movie_count.FindValue (myMovieTable.rows[i]->owner_id, count);
      count++;
      user_movie_count.Remove (myMovieTable.rows[i]->owner_id);
      user_movie_count.Insert (myMovieTable.rows[i]->owner_id, count);

      /* Update the number of movies per format count. */
      count = 0;
      format_movie_count.FindValue (myMovieTable.rows[i]->format_id, count);
      count++;
      format_movie_count.Remove (myMovieTable.rows[i]->format_id);
      format_movie_count.Insert (myMovieTable.rows[i]->format_id, count);

      /* Set colour */
      myMovieList.set_background (i, colours[i & 1]);
    }
  
  entries = myUserTable.rows.Entries ();
  for (i = 0; i < entries; i++)
    {
      /* Fetch number of movies for this user. */
      count = 0;
      user_movie_count.FindValue (myUserTable.rows[i]->id, count);
      sprintf (buf, "%d", count);
      
      row[0] = myUserTable.rows[i]->name;
      row[1] = buf;
      row[2] = 0;
      myUserList.Append (row);
      
      if (myUserTable.rows[i]->display)
	myUserList.cell(i, 2).set_pixmap (myOkPixmap, myOkBitmap);
      else
	myUserList.cell(i, 2).set_pixmap (myCancelPixmap, myCancelBitmap);

      /* Set colour */
      myUserList.set_background (i, colours[i & 1]);
    }
  
  entries = myFormatTable.rows.Entries ();
  for (i = 0; i < entries; i++)
    {
      /* Fetch number of movies for this format. */
      count = 0;
      format_movie_count.FindValue (myFormatTable.rows[i]->id, count);
      sprintf (buf, "%d", count);
      
      row[0] = myFormatTable.rows[i]->name;
      row[1] = buf;
      row[2] = 0;
      myFormatList.Append (row);

      if (myFormatTable.rows[i]->display)
	myFormatList.cell(i, 2).set_pixmap (myOkPixmap, myOkBitmap);
      else
	myFormatList.cell(i, 2).set_pixmap (myCancelPixmap, myCancelBitmap);
      
      /* Set colour */
      myFormatList.set_background (i, colours[i & 1]);
    }
  
  /* Thaw lists. */
  myMovieList.thaw ();
  myUserList.thaw ();
  myFormatList.thaw ();
}


void
UiMain::Update (int dummy)
{
  TValListT<int> *selection;
  int row, page = myNotebook.get_current_page_num ();
  bool update_done = 0;
  
  selection = myUiLists[page]->GetSelectedRows ();
  while (selection->Entries ())
    {
      row = selection->Remove();
      
      if (page == 0)
	{
	  myMovieForm.SetData (*myMovieTable.rows[row],
			       myUserTable, myFormatTable);
	  
	  myMovieForm.show ();
	  myMovieForm.Block ();
	  
	  if (myMovieForm.OkPressed ())
	    {
	      myMovieForm.GetData (*myMovieTable.rows[row],
				   myUserTable, myFormatTable);
	      
	      myMovieTable.Update (*myMovieTable.rows[row]);
	      DisplayTableErrors (&myMovieTable);
	      update_done = true;
	    }
	  else
	    break;
	}
      else if (page == 1)
	{
	  myUserForm.SetData (*myUserTable.rows[row]);
	  myUserForm.show ();
	  myUserForm.Block ();
	  
	  if (myUserForm.OkPressed ())
	    {
	      myUserForm.GetData (*myUserTable.rows[row]);
	      myUserTable.Update (*myUserTable.rows[row]);
	      DisplayTableErrors (&myUserTable);
	      update_done = true;
	    }
	  else
	    break;
	}
      else
	{
	  myFormatForm.SetData (*myFormatTable.rows[row]);
	  myFormatForm.show ();
	  myFormatForm.Block ();
	  
	  if (myFormatForm.OkPressed ())
	    {
	      myFormatForm.GetData (*myFormatTable.rows[row]);
	      myFormatTable.Update (*myFormatTable.rows[row]);
	      DisplayTableErrors (&myFormatTable);
	      update_done = true;
	    }
	  else
	    break;
	}
    }
  
  if (update_done)
    LoadData ();
  
  delete selection;
}


void
UiMain::Insert (int dummy)
{
  int page = myNotebook.get_current_page_num (), i;
  static TString prev_format_id;
  static TString prev_owner_id;
  static TString prev_cuser_id;
  
  if (page == 0)
    {
      /* Must have formats and users! */
      if (! myUserTable.rows.Entries () ||
	  ! myFormatTable.rows.Entries ())
	return;
      
      MovieRow row;
      
      row.cuser_id = myUserTable.rows[0]->id;
      row.format_id = myFormatTable.rows[0]->id;
      
      /* Set default choices based on previous insert. */
      if (myFormatTable.map.FindValue (prev_format_id, i))
	{
	  row.format_id = myFormatTable.rows[i]->id;
	  row.format_index = i;
	}
      else
	row.format_id = myFormatTable.rows[0]->id;
      
      if (myUserTable.map.FindValue (prev_owner_id, i))
	{
	  row.owner_id = myUserTable.rows[i]->id;
	  row.owner_index = i;
	}
      else
	row.owner_id = myUserTable.rows[0]->id;
      
      if (myUserTable.map.FindValue (prev_cuser_id, i))
	{
	  row.cuser_id = myUserTable.rows[i]->id;
	  row.cuser_index = i;
	}
      else
	row.cuser_id = myUserTable.rows[0]->id;
      
      /* Update form */
      myMovieForm.SetData (row, myUserTable, myFormatTable);
      myMovieForm.show ();
      myMovieForm.Block ();
      
      if (myMovieForm.OkPressed ())
	{
	  myMovieForm.GetData (row, myUserTable, myFormatTable);
	  myMovieTable.Insert (row);
	  DisplayTableErrors (&myMovieTable);
	  
	  prev_format_id = row.format_id;
	  prev_owner_id = row.owner_id;
	  prev_cuser_id = row.cuser_id;
	  
	  LoadData ();
	}
    }
  else if (page == 1)
    {
      NameRow row (true);
      
      myUserForm.SetData (row);
      myUserForm.show ();
      myUserForm.Block ();
      
      if (myUserForm.OkPressed ())
	{
	  myUserForm.GetData (row);
	  myUserTable.Insert (row);
	  DisplayTableErrors (&myUserTable);
	  LoadData ();
	}
    }
  else
    {
      NameRow row (true);
      
      myFormatForm.SetData (row);
      myFormatForm.show ();
      myFormatForm.Block ();
      
      if (myFormatForm.OkPressed ())
	{
	  myFormatForm.GetData (row);
	  myFormatTable.Insert (row);
	  DisplayTableErrors (&myFormatTable);
	  LoadData ();
	}
    }
}


void
UiMain::Delete (int dummy)
{
  int page = myNotebook.get_current_page_num ();
  
  page = myNotebook.get_current_page_num ();
  
  if (myUiLists[page]->GetNumSelectedRows ())
    {
      UiQuestionBox qb ("Do you want to delete the selected entries");
      
      qb.Block ();
      if (qb.OkPressed ())
	{
	  TValListT<int> *selection = myUiLists[page]->GetSelectedRows ();
	  int row;
	  
	  while (selection->Entries ())
	    {
	      row = selection->Remove ();
	      myTables[page]->Delete (row);
	      DisplayTableErrors (myTables[page]);
	    }
	  
	  delete (selection);
	  LoadData ();
	}
    }
}


void
UiMain::Select (int dummy)
{
  /* Reload all data from DB, this is expensive. */
  LoadData ();
}


void
UiMain::Toggle (int dummy)
{
  TValListT<int> *selection;
  int row, page = myNotebook.get_current_page_num ();
  bool update_done = false;
  
  selection = myUiLists[page]->GetSelectedRows ();
  while (selection->Entries ())
    {
      row = selection->Remove();
      
      if (page == 1)
	{
	  myUserTable.rows[row]->display ^= 1;
	  update_done = true;
	}
      else if (page == 2)
	{
	  myFormatTable.rows[row]->display ^= 1;
	  update_done = true;
	}
    }
  
  if (update_done)
    LoadData ();
  
  delete selection;
}


void
UiMain::SortOrder (int dummy)
{
  /* Just toggle the variable */
  mySortByInsertOrder = !mySortByInsertOrder;
  
  /* Reload all data from DB, this is expensive. */
  LoadData ();
}


void
UiMain::About (int dummy)
{
  myAboutWindow.show ();
}


void
UiMain::DisplayTableErrors (Table *table)
{
  while (table->HasErrors ())
    UiMain::Message (table->PopError ());
}


gint
UiMain::delete_event_impl (GdkEventAny*)
{
  Gtk::Main::quit();
  return 0;
}
