#ifndef _UI_MAIN_HH
#define _UI_MAIN_HH

#include <gtk--/menubar.h>
#include <gtk--/notebook.h>
#include <gtk--/scrolledwindow.h>
#include <gtk--/statusbar.h>
#include <gtk--/window.h>
#include "TString.hh"
#include "UiList.hh"
#include "UiAbout.hh"
#include "UiMovieForm.hh"
#include "UiNameForm.hh"
#include "MovieTable.hh"


/**
 * Main MovieDB window.
 * This window handles all other sub windows.
 */

class UiMain : public Gtk::Window
{
public:
  
  UiMain ();
  
  /**
   * Load data from database and uppdate lists.
   */
  void LoadData ();
  
  /**
   * Push message onto statusbar.
   */
  static void Message (const TString& str)
  {
    if (myStatusbar)
      myStatusbar->push (1, str.Data ());
  }
  
private:
  
  /**
   * Update menu entry callback.
   */
  void Update (int dummy);

  /**
   * Insert menu entry callback.
   */
  void Insert (int dummy);
  
  /**
   * Delete menu entry callback.
   */
  void Delete (int dummy);
  
  /**
   * Select menu entry callback.
   */
  void Select (int dummy);
  
  /**
   * Toggle menu entry callback.
   */
  void Toggle (int dummy);
  
  /**
   * Toggle the sort order.
   */
  void SortOrder (int dummy);

  /**
   * About window callback.
   */
  void About (int dummy);
  
  /**
   * Display any database table errors in status bar.
   */
  void DisplayTableErrors (Table *table);
  
  /**
   * Destroy window on delete.
   */
  gint delete_event_impl (GdkEventAny*);
  
  
  UiMovieForm myMovieForm;
  UiNameForm myUserForm;
  UiNameForm myFormatForm;
  
  UiAbout myAboutWindow;
  
  UiList myMovieList;
  UiList myUserList;
  UiList myFormatList;
  
  Gtk::Notebook myNotebook;
  
  Gdk_Pixmap myOkPixmap;
  Gdk_Bitmap myOkBitmap;
  
  Gdk_Pixmap myCancelPixmap;
  Gdk_Bitmap myCancelBitmap;
  
  MovieTable myMovieTable;
  UserTable myUserTable;
  FormatTable myFormatTable;
  
  UiList *myUiLists[3];  
  Table *myTables[3];
  
  /* If to sort by insert order */
  bool mySortByInsertOrder;
  
  static Gtk::Statusbar *myStatusbar;
};

#endif // _UI_MAIN_HH
