/* MovieDB HTML Dumper */

#include <TString.hh>
#include <TFile.hh>
#include <TDBConnectionManager.hh>
#include "MovieTable.hh"

// Display this many movies
#define NUM_LATEST_MOVIES  10


void
usage ()
{
  fprintf (stderr, "usage: moviedb_html <target file>\n");
  exit (1);
}


int
main (int argc, char **argv)
{
  if (argc != 2)
    usage ();
  
  // MovieDB tables
  MovieTable mtable;
  UserTable utable;
  FormatTable ftable;
  
  // Some HTML tags
  TString h1_beg ("<h1>");
  TString h1_end ("</h1>");
  TString td_beg ("<td>");
  TString td_end ("</td>");
  TString tr_beg ("<tr>");
  TString tr_end ("</tr>");
  TString table_end ("</table>");
  
  TString new_line (10);
  
  // Target and template HTML files
  TFile target (argv[1], "w");
  TFile header (DATADIR "/header.html", "r");
  TFile table_header1 (DATADIR "/table_header1.html", "r");
  TFile table_header2 (DATADIR "/table_header2.html", "r");
  TFile trailer (DATADIR "/trailer.html", "r");
  TFile *files[5] = { &target, &header, &table_header1,
		      &table_header2, &trailer };
  
  // Check all files
  for (int i = 0; i < 5; i++)
    {
      if (! files[i]->IsValid())
	{
	  fprintf (stderr, "E: Could not open %s\n",
		   (const char *)files[i]->Path());
	  exit (1);
	}
    }
  
  /* Set default database login information. */
  TDBConnectionManager::SetLogin ("postgres", "", "moviedb", "", "");
  
  target.Write (header);
  
  utable.Select ();
  ftable.Select ();
  
  // Latest Movies
  mtable.Select (utable, ftable, true);
  int movies = mtable.rows.Entries ();
  
  if (movies)
    {
      target.Write (h1_beg + "Latest Movies" + h1_end + new_line);
      target.Write (table_header1);
      
      if (movies > NUM_LATEST_MOVIES)
	movies = NUM_LATEST_MOVIES;
      
      for (int i = 0; i < movies; i++)
	{
	  target.Write (tr_beg);
	  target.Write (td_beg + mtable.rows[i]->title + td_end);
	  target.Write (td_beg + mtable.rows[i]->format + td_end);
	  target.Write (td_beg + mtable.rows[i]->owner + td_end);
	  target.Write (tr_end + new_line);
	}
      
      target.Write (table_end + new_line);
    }
  
  // All formats one by one
  int formats = ftable.rows.Entries ();
  
  // Clear all formats for use in select
  for (int i = 0; i < formats; i++)
    ftable.rows[i]->display = false;
  
  for (int i = 0; i < formats; i++)
    {
      // Select one format at a time
      ftable.rows[i]->display = true;
      mtable.Select (utable, ftable, false);
      
      movies = mtable.rows.Entries ();
      if (movies)
	{
	  target.Write (h1_beg + ftable.rows[i]->name + h1_end + new_line);
	  table_header2.Seek (0);
	  target.Write (table_header2);
	  
	  for (int i = 0; i < movies; i++)
	    {
	      target.Write (tr_beg);
	      target.Write (td_beg + mtable.rows[i]->title + td_end);
	      target.Write (td_beg + mtable.rows[i]->owner + td_end);
	      target.Write (tr_end + new_line);
	    }
	  
	  target.Write (table_end + new_line);
	}
      
      ftable.rows[i]->display = false;
    }
  
  target.Write (new_line);
  target.Write (trailer);
  target.Close ();
  
  /* Close database connections. */
  TDBConnectionManager::Close ();
  
  return 0;
}
